from pathlib import Path
from typing import Literal
import time
import os

from ..Butils import B_Color

def color_wrap(string:str, color:B_Color):
    return color.value + string + B_Color.RESET.value

class B_Writer:
    def __init__(
        self,
        path:Path,
        mode:Literal["w", "a"] = "w",
        time_file:bool = False,
        color:B_Color = B_Color.RESET,
    ):
        '''
        :param path: 日志保存路径
        :param time_file: 是否输出时间
        '''
        super().__init__()
        self.path = Path(path)
        self.mode = mode
        self.time_file = time_file

        self.color = color

        self.f = None
        self.setFile(self.path, self.mode, self.time_file)

    def setFile(self, file: Path, mode: Literal["w", "a"] = "w", ifTime=False):
        '''
        设置 file的path 以及 writer的ifTime
        :param file: 设置log路径
        :param ifTime:
        :return:
        '''
        if self.f is not None:
            self.f.close()
        self.path = Path(file)
        self.time_file = ifTime
        self.__createDir(self.path)
        self.f = open(self.path, mode, encoding="utf-8")

    def clearFile(self):
        '''
        清空内容
        '''
        assert self.f is not None, "请先调用setFile方法"
        self.f.close()
        self.f = open(self.path, 'w', encoding="utf-8")

    def closeFile(self):
        '''
        关闭log
        '''
        if self.f:
            self.f.close()
            self.f = None

    def toCmd(self, *args:str, ifTime:bool=False, color:B_Color = None):
        '''
        打印到terminal
        '''
        t = ''
        if ifTime:
            t = time.strftime("%Y-%m-%d %H:%M:%S ##### ", time.localtime())

        if color is None:
            for string in args:
                print(color_wrap(f"{t}{string}", self.color))
        else:
            for string in args:
                print(color_wrap(f"{t}{string}", color))

    def toFile(self, *args:str, ifTime:bool=None):
        '''
        写入到文件内
        '''
        assert self.f is not None, "请先调用setFile方法"

        t = ''
        if ifTime == False: # 为了使False时不管self.time_file
            pass
        elif ifTime==True or self.time_file==True:
            t = time.strftime("%Y-%m-%d %H:%M:%S ##### ", time.localtime())

        for string in args:
            self.f.write(f"{t}{string}\n")
        self.f.flush()

    def toBoth(self, *args:str, file_time:bool=None, cmd_time:bool=False, color:B_Color = None):
        '''
        同时写入到文件和terminal
        :param string:
        :param color:
        :return:
        '''
        for string in args:
            self.toFile(str(string), file_time)
            self.toCmd(str(string), cmd_time, color)
    def __call__(self, *args:str, file_time:bool=None, cmd_time:bool=False, color:B_Color = None):
        self.toBoth(*args, file_time, cmd_time, color)

    def __createDir(self, path):
        # 获取到该文件的文件夹
        dir = path.parents[0]
        os.makedirs(dir, exist_ok=True)

    def __exit__(self):
        self.closeFile()

if __name__ == '__main__':
    B_Writer()


import pandas as pd
from typing import Literal

from .. import B_os
# from byzh.core import B_os

class B_Record2d:
    def __init__(self, csv_path, mode:Literal["w", "a", "r"]="r", title:str=None):
        self.csv_path = csv_path
        self.mode = mode
        self.title = title

        B_os.makedirs(csv_path)
        self.data = pd.DataFrame()
        if mode == "w":
            B_os.rm(csv_path)
            self.__read()
        elif mode == "a":
            self.__read()
        elif mode == "r":
            self.__read()

        if self.data.index.name is None:
            self.data.index.name = title
    # 支持 recorder[row, col] 访问
    def __getitem__(self, key):
        if isinstance(key, tuple) and len(key) == 2: # 读值操作
            row, col = key
            row, col = str(row), str(col)
            return self.data.loc[row, col]
        if isinstance(key, str): # 读行操作
            row = str(key)
            return self.data.loc[row, :]

    # 支持 recorder[row, col] = value
    def __setitem__(self, key, value):
        assert self.mode!="r", "不允许读模式下赋值"

        if isinstance(key, tuple) and len(key) == 2: # 赋值操作
            row, col = key
            row, col, value = str(row), str(col), str(value)
            self.data.loc[row, col] = value
            self.__save()
        if isinstance(key, str): # 赋值行操作
            assert type(value) == pd.Series, "仅支持 Series 赋值"
            key = str(key)
            self.data.loc[key, :] = value.copy()
            self.__save()

    # 复制一行
    def copy_row(self, new_row, origin_row):
        new_row, origin_row = str(new_row), str(origin_row)

        if origin_row not in self.data.index:
            raise KeyError(f"源行 {origin_row} 不存在")

        # 深拷贝一份行数据
        self.data.loc[new_row] = self.data.loc[origin_row].copy()

        self.__save()
        return self

    # 转置
    def T(self):
        self.data = self.data.T.copy()
        self.__save()
        return self

    def write(self, row, col, value):
        self[row, col] = value
        return self

    def get(self, row, col):
        return self[row, col]

    def get_str(self, row, col) -> str:
        return str(self[row, col])

    def get_int(self, row, col) -> int:
        return int(self[row, col])

    def get_float(self, row, col) -> float:
        return float(self[row, col])

    def get_bool(self, row, col) -> bool:
        result = self.get_str(row, col)
        if result.lower() in ("true", "1"):
            return True
        elif result.lower() in ("false", "0"):
            return False
        else:
            raise ValueError(f"无法转换为布尔值 -> {result}")

    def __read(self):
        try:
            self.data = pd.read_csv(self.csv_path, index_col=0)
        except (FileNotFoundError, pd.errors.EmptyDataError):
            self.data = pd.DataFrame()

    def __save(self, csv_path=None):
        if csv_path is None:
            csv_path = self.csv_path
        self.data.to_csv(csv_path, index=True, encoding='utf-8-sig')

    def __str__(self):
        return str(self.data)


if __name__ == '__main__':
    csv_file = "test_data.csv"

    recorder = B_Record2d(csv_file, mode="w", title='x\y')

    # 用索引方式赋值
    recorder["awa", "OvO"] = 10
    recorder["awa", "TwT"] = 20
    recorder["qwq", "OvO"] = 30
    recorder["qwq", "TwT"] = 40

    recorder['aaa'] = recorder['qwq']

    print("当前内容：")
    print(recorder)

    # 用索引方式读取
    print("awa, OvO =", recorder["awa", "OvO"])

    recorder = B_Record2d(csv_file, mode="r")
    print(recorder)


from .text_style import B_Color, B_Appearance, B_Background
from .timer import B_Timer
from .decorator import b_validate_params
from .project import b_create_project

__all__ = [
    'B_Color', 'B_Appearance', 'B_Background',
    'B_Timer',
    'b_validate_params',
    'b_create_project'
]


import os
from pathlib import Path

#### 项目结构 ####
PROJECT_STRUCTURE = {
    # 模块文件
    "models": [
        "__init__.py",
    ],
    "training": [
        "__init__.py",
    ],
    "visualization": [
        "__init__.py",
    ],
    "utils": [
        "__init__.py",
    ],
    "configs": [
        "config.yaml",
    ],
    "scripts": [
        "run_pipeline_1.py",
        "run_pipeline_2.py",
        "run_pipeline_3.py",
    ],

    # 非模块文件
    "data": [
        "README.md",
    ],
    "logs": [],
    "outputs": [],
}

ROOT_FILES = [
    ".gitignore",
    "README.md",
    "requirements.txt",
]

#### 具体内容 ####
GITIGNORE_CONTENT = \
"""
# Python
__pycache__/
*.pyc
*.pyo
*.pyd

# virtual env
.venv/
venv/
env/

# logs
logs/

# outputs
outputs/

# dataset
data/*

# IDE
.vscode/
.idea/

# mac
.DS_Store
"""

def _make(file_path, content=None, overwrite=False):
    # 删去旧的, 创建新的
    if os.path.exists(file_path) and overwrite:
        print(f"Overwriting {file_path}...")
        file_path.unlink()
        if content:
            file_path.write_text(content.strip() + "\n")
        else:
            file_path.touch(exist_ok=True)
    # 保持不变
    elif os.path.exists(file_path):
        pass
    # 创建新的
    else:
        if content:
            file_path.write_text(content.strip() + "\n")
        else:
            file_path.touch(exist_ok=True)


def b_create_project(project_dir: Path, overwrite=False):
    """
    创建项目目录结构
    """
    project_dir = Path(project_dir)

    # 创建根目录内文件夹
    for folder, files in PROJECT_STRUCTURE.items():
        folder_path = project_dir / folder
        folder_path.mkdir(parents=True, exist_ok=True)

        for file in files:
            file_path = (folder_path / file)
            _make(file_path, overwrite=overwrite)

    # 创建根目录内文件
    for file in ROOT_FILES:
        file_path = project_dir / file

        if file == ".gitignore":
            _make(file_path, content=GITIGNORE_CONTENT, overwrite=overwrite)
        else:
            _make(file_path, overwrite=overwrite)


def main():
    base_path = Path(".").resolve()
    print(f"Creating project structure in: {base_path}")

    b_create_project(base_path)
    print("Project structure created successfully!")


if __name__ == "__main__":
    main()


import os
import zipfile
import time
from pathlib import Path

from .. import B_os
from ..Btqdm import B_Tqdm
from ..Butils import b_validate_params

@b_validate_params({
    "black_dirnames": {"examples": ["__pycache__", ".git", ".idea"]},
    "black_filenames": {"examples": [".gitignore", "README.md"]},
    "black_stems": {"examples": ["README"]},
    "black_exts": {"examples": [".csv", ".npy", ".pt", ".pth"]},
    "white_filepath": {"examples": ["./.gitignore", "./README.md"]},
    "white_dirpath": {"examples": [r"E:\byzh_workingplace\...\文件夹1"]},
})
def b_archive_zip(
    source_path,
    output_path:str|Path=None,
    black_dirnames:list[str]=None,
    black_filenames:list[str]=None,
    black_stems:list[str]=None,
    black_exts:list[str]=None,
    white_filepath:list[str]=None,
    white_dirpath:list[str]=None,
    contain_empty_folder:bool=True,
):
    '''
    压缩文件夹，排除 指定文件夹and指定后缀文件

    name代表前缀, ext代表后缀(包括.)

    :param source_path:
    :param output_path: 如果不传入, 则默认为source_path同名同路径的zip文件
    :param black_dirnames: 黑名单-文件夹名
    :param black_filenames: 黑名单-文件全名
    :param black_stems: 黑名单-文件名(不含后缀)
    :param black_exts: 黑名单-文件后缀(包括.)
    :param white_filepath: 白名单-文件全名
    :param white_dirpath: 白名单-文件夹路径
    :param contain_empty_folder: 是否包含空文件夹
    :return:
    '''
    if output_path is None:
        if os.path.isdir(source_path):
            output_path = source_path + '.zip'
        else:
            output_path = os.path.splitext(source_path)[0] + '.zip'

    if os.path.exists(output_path):
        print(f"检测到{output_path}已存在, 将在3秒后删除...")
        time.sleep(3)
        B_os.rm(output_path)
        print(f"{output_path}已删除")
    def get_lst(lst):
        return [] if lst is None else lst
    black_dirnames = get_lst(black_dirnames)
    black_filenames = get_lst(black_filenames)
    black_stems = get_lst(black_stems)
    black_exts = get_lst(black_exts)
    white_filepath = get_lst(white_filepath)
    white_dirpath = get_lst(white_dirpath)

    my_tqdm = B_Tqdm(prefix='[b_archive_zip]')

    with zipfile.ZipFile(output_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
        # 如果传入的是文件:
        if os.path.isfile(source_path):
            arcname = os.path.basename(source_path)
            zipf.write(source_path, arcname)
            my_tqdm.update(1)
            return

        # 如果传入的是文件夹:
        if os.path.isdir(source_path):
            for root, dirs, files in B_os.walk(source_path,
                                               black_dirnames=black_dirnames,
                                               black_filenames=black_filenames,
                                               black_stems=black_stems,
                                               black_exts=black_exts):

                # 压缩文件:
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, source_path) # 相对于source_path的相对路径
                    zipf.write(file_path, arcname)
                    my_tqdm.update(1)
                # 压缩文件夹:
                for i, dir in enumerate(dirs):
                    if len(os.listdir(os.path.join(root, dir))) == 0:
                        if not contain_empty_folder:
                            dirs.pop(i)
                            continue
                    dir_path = os.path.join(root, dir)
                    arcname = os.path.relpath(dir_path, source_path) # 相对于source_path的相对路径
                    zipf.write(dir_path, arcname)
                    my_tqdm.update(1)

        # 白名单-文件:
        for filepath in white_filepath:
            arcname = os.path.relpath(filepath, source_path)
            zipf.write(filepath, arcname)
            my_tqdm.update(1)

        # 白名单-文件夹:
        for dirpath in white_dirpath:
            for root, dirs, files in B_os.walk(dirpath):
                # 压缩文件:
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, source_path)  # 相对于source_path的相对路径
                    zipf.write(file_path, arcname)
                    my_tqdm.update(1)
                # 压缩文件夹:
                for i, dir in enumerate(dirs):
                    if len(os.listdir(os.path.join(root, dir))) == 0:
                        if not contain_empty_folder:
                            dirs.pop(i)
                            continue
                    dir_path = os.path.join(root, dir)
                    arcname = os.path.relpath(dir_path, source_path)  # 相对于source_path的相对路径
                    zipf.write(dir_path, arcname)
                    my_tqdm.update(1)

if __name__ == '__main__':
    # 不包含空文件夹:`文件夹111`
    B_os.makedirs('根文件夹/文件夹1/文件夹11/文件夹111')
    contain_empty_folder = False

    # 不包含`文件夹12`
    B_os.makefile('根文件夹/文件夹1/文件夹12/file1.txt')
    B_os.makefile('根文件夹/文件夹1/文件夹12/文件夹123/file2.txt')
    B_os.makefile('根文件夹/文件夹1/文件夹12/文件夹1234/文件夹12345/file3.txt')
    black_dirnames = ['文件夹12']

    # 不包含`file4.txt`, 且因此`文件夹21`变为空文件夹, 因此不包含
    B_os.makefile('根文件夹/文件夹2/文件夹21/file4.txt')
    black_filenames = ['file4.txt']

    # 不包含`awa.txt`, `awa.html`
    B_os.makefile('根文件夹/文件夹3/awa.txt')
    B_os.makefile('根文件夹/文件夹3/文件夹31/awa.html')
    black_stems = ['awa']

    # 不包含`file5.csv`
    B_os.makefile('根文件夹/文件夹4/file5.csv')
    black_exts = ['.csv']

    white_filepath = ['根文件夹/文件夹1/文件夹12/文件夹1234/文件夹12345/file3.txt']
    white_dirpath = ['根文件夹/文件夹1/文件夹12/文件夹123']

    b_archive_zip(
        source_path=r'根文件夹',
        output_path=r'awaqwq.zip',
        black_dirnames=black_dirnames,
        black_filenames=black_filenames,
        black_stems=black_stems,
        black_exts=black_exts,
        white_filepath=white_filepath,
        white_dirpath=white_dirpath,
        contain_empty_folder=contain_empty_folder,
    )